import path from "node:path";
import {
  pico,
  minimist,
  consola,
  inquirer,
  getMonorepoPkgListInfo
} from "@mango-scripts/utils";
import {
  confirmRegistry,
  confirmGitBranch,
  confirmWorktreeEmpty,
  confirmNpmLoggedIn
} from "./confirmEnv.mjs";
import { updateVersion, updateDepVersion, updateChangelog } from "./update.mjs";
import { getDiffPkgList } from "./diff.mjs";
import { setPkgTargetVersion } from "./version.mjs";
import { run, step } from "../../utils/index.mjs";
const publish = async (tag) => {
  if (!tag || !tag.includes("@")) throw new Error("\u65E0\u6548\u7684\u53D1\u5E03TAG");
  const [pkgName, version] = tag.split("@");
  const rawPkgName = pkgName.split("/")[1];
  const releaseType = version.includes("beta") ? "beta" : version.includes("alpha") ? "alpha" : void 0;
  step(`\u53D1\u5E03 ${pkgName} \u4E2D...`);
  const publicArgs = ["publish", "--access", "public"];
  if (releaseType) {
    publicArgs.push(`--tag`, releaseType);
  }
  publicArgs.push(`--no-git-checks`);
  await run("pnpm", publicArgs, {
    cwd: path.join(process.cwd(), `./packages/${rawPkgName}`)
  });
};
const release = async () => {
  if (!await confirmGitBranch() || !await confirmWorktreeEmpty()) return;
  const { publishType } = await inquirer.prompt([
    {
      type: "list",
      name: "publishType",
      message: "\u9009\u62E9\u53D1\u5E03\u65B9\u5F0F",
      choices: [
        { name: "\u8FDC\u7A0BCI\u53D1\u5E03", value: "remote" },
        { name: "\u672C\u5730\u6253\u5305\u53D1\u5E03", value: "local" }
      ]
    }
  ]);
  step(`\u5DF2\u9009\u62E9 ${publishType === "local" ? "[\u672C\u5730\u6253\u5305\u53D1\u5E03]" : "[\u8FDC\u7A0BCI\u53D1\u5E03]"}`);
  if (publishType === "local") {
    if (!await confirmRegistry() || !await confirmNpmLoggedIn()) return;
  } else {
    consola.info(
      pico.red(`\u4F7F\u7528[\u8FDC\u7A0BCI\u53D1\u5E03]\u524D\uFF0C\u4F60\u9700\u8981\u786E\u8BA4\u8FDC\u7A0BCI\u4E0A\u5DF2\u914D\u7F6E publish \u547D\u4EE4`)
    );
  }
  const pkgList = await getMonorepoPkgListInfo();
  const diffPkgList = await getDiffPkgList(pkgList);
  const { selectedPkg } = await inquirer.prompt([
    {
      type: "list",
      name: "selectedPkg",
      message: "\u9009\u62E9\u5F85\u53D1\u5E03\u7684\u5305",
      choices: diffPkgList.map((pkg) => {
        const commitInfo = pkg?.commitList?.length ? pico.green(` [\u65B0\u589E ${pkg?.commitList?.length} commit]`) : "";
        const depChangedInfo = pkg.changedDep && pkg.changedDep?.length > 0 ? pico.magenta(
          ` [\u4F9D\u8D56\u66F4\u65B0][${pkg.changedDep.map((it) => `${it.pkg.pkgName}<${it.level.join(",")}\u7EA7\u4F9D\u8D56>`).join(", ")}]`
        ) : "";
        return {
          name: `${pkg.pkgName}${commitInfo}${depChangedInfo}`,
          value: pkg
        };
      }),
      loop: false,
      pageSize: 100
    }
  ]);
  if (!selectedPkg) return;
  const { pkgName, pkgJsonFilePath, pkgDirPath, commitList } = selectedPkg;
  console.log();
  commitList?.length && consola.info(`\u6700\u8FD1 commit\uFF1A
${commitList.join("\n")}  
      `);
  const pkgTargetVersion = await setPkgTargetVersion(selectedPkg);
  if (!pkgTargetVersion) return;
  const tag = `${pkgName}@${pkgTargetVersion}`;
  const { isTagRight } = await inquirer.prompt([
    {
      type: "confirm",
      name: "isTagRight",
      message: `\u5F53\u524D\u7684\u53D1\u5E03TAG\uFF1A ${pico.yellow(tag)}\uFF0C\u786E\u5B9A\uFF1F`
    }
  ]);
  if (!isTagRight) return;
  const { isPublishAlready } = await inquirer.prompt([
    {
      type: "confirm",
      name: "isPublishAlready",
      message: `\u786E\u5B9A\u5F00\u59CB\u53D1\u5E03\uFF1F`,
      default: false
    }
  ]);
  if (!isPublishAlready) {
    console.error("\u5DF2\u53D6\u6D88\u53D1\u5E03");
    return;
  }
  step("\u5347\u7EA7 package.json \u7248\u672C\u53F7...");
  await updateVersion(pkgJsonFilePath, pkgTargetVersion);
  step("\u5347\u7EA7 package.json \u4F9D\u8D56\u7248\u672C\u53F7...");
  await updateDepVersion(pkgJsonFilePath, pkgList);
  step("\u751F\u6210 changelog...");
  await updateChangelog(pkgDirPath, pkgName);
  const { stdout } = await run("git", ["diff"], { stdio: "pipe" });
  if (stdout) {
    step("\u63D0\u4EA4version\u3001changelog\u7B49\u4EE3\u7801\u53D8\u66F4...");
    await run("git", ["add", "-A"]);
    await run("git", ["commit", "-m", `feat: release ${tag}`]);
    await run("git", ["tag", tag]);
  } else {
    consola.info("\u65E0\u53D8\u66F4");
    return;
  }
  step("\u63A8\u9001\u5230\u8FDC\u7A0B\u5206\u652F...");
  await run("git", ["push", "origin", `refs/tags/${tag}`]);
  await run("git", ["push"]);
  if (publishType === "local") {
    step("\u672C\u5730\u6253\u5305\u4E2D...");
    await run("pnpm", ["run", "build"], { cwd: process.cwd() });
    step("\u4E0A\u4F20\u5230 npm...");
    publish(tag);
  } else {
    consola.success(
      pico.green(
        `
    \u63A8\u9001\u5230\u8FDC\u7A0B\u4ED3\u5E93\u6210\u529F\uFF0C\u8FDC\u7A0B\u6253\u5305\u90E8\u7F72\u5373\u5C06\u542F\u52A8\uFF0C\u8BF7\u5728\u8FDC\u7A0B\u4ED3\u5E93\u67E5\u770B\u72B6\u6001
    `
      )
    );
  }
};
const releasePackage = async () => {
  const { tag } = minimist(process.argv.slice(2));
  tag ? publish(tag) : release();
};
var releasePackage_default = releasePackage;
export {
  releasePackage_default as default,
  publish,
  release
};
